module TurbaKit
  class Error < StandardError
    def self.from_response(body, status)
      if klass =  case status.to_i
                  when 400      then TurbaKit::BadRequest
                  when 401      then TurbaKit::Unauthorized
                  when 403      then TurbaKit::Forbidden
                  when 404      then TurbaKit::NotFound
                  when 405      then TurbaKit::MethodNotAllowed
                  when 406      then TurbaKit::NotAcceptable
                  when 409      then TurbaKit::Conflict
                  when 415      then TurbaKit::UnsupportedMediaType
                  when 422      then TurbaKit::UnprocessableEntity
                  when 400..499 then TurbaKit::ClientError
                  when 500      then TurbaKit::InternalServerError
                  when 501      then TurbaKit::NotImplemented
                  when 502      then TurbaKit::BadGateway
                  when 503      then TurbaKit::ServiceUnavailable
                  when 500..599 then TurbaKit::ServerError
                  end
        klass.new(body, status)
      end
    end

    def initialize(body, status)
      message = {
        status: status,
        error: body['error']
      }

      super(Oj.dump(message, mode: :compat)) 
      raise self
    end

  end

  # Raised on errors in the 400-499 range
  class ClientError < Error; end
  class BadRequest < ClientError; end
  class Unauthorized < ClientError; end
  class Forbidden < ClientError; end
  class NotFound < ClientError; end
  class MethodNotAllowed < ClientError; end
  class NotAcceptable < ClientError; end
  class Conflict < ClientError; end
  class UnsupportedMediaType < ClientError; end
  class UnprocessableEntity < ClientError; end
  class ServerError < Error; end
  class InternalServerError < ServerError; end
  class NotImplemented < ServerError; end
  class BadGateway < ServerError; end
  class ServiceUnavailable < ServerError; end
  class MissingContentType < ArgumentError; end
  class ApplicationCredentialsRequired < StandardError; end
end
